# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.api.tests_api import TestsApi


class TestTestsApi(unittest.TestCase):
    """TestsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = TestsApi()

    def tearDown(self) -> None:
        pass

    def test_settings_teststatuses_get(self) -> None:
        """Test case for settings_teststatuses_get

        """
        pass

    def test_test_get(self) -> None:
        """Test case for test_get

        """
        pass

    def test_test_key_preconditions_get(self) -> None:
        """Test case for test_key_preconditions_get

        """
        pass

    def test_test_key_testexecutions_get(self) -> None:
        """Test case for test_key_testexecutions_get

        """
        pass

    def test_test_key_testplans_get(self) -> None:
        """Test case for test_key_testplans_get

        """
        pass

    def test_test_key_testruns_get(self) -> None:
        """Test case for test_key_testruns_get

        """
        pass

    def test_test_key_testsets_get(self) -> None:
        """Test case for test_key_testsets_get

        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.v10_api_test_get200_response_inner_definition_one_of_steps_inner_step import V10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep

class TestV10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep(unittest.TestCase):
    """V10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> V10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep:
        """Test V10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `V10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep`
        """
        model = V10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep()
        if include_optional:
            return V10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep(
                raw = '',
                rendered = ''
            )
        else:
            return V10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep(
        )
        """

    def testV10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep(self):
        """Test V10ApiTestGet200ResponseInnerDefinitionOneOfStepsInnerStep"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.xray_json_import_execution_tests_inner import XrayJsonImportExecutionTestsInner

class TestXrayJsonImportExecutionTestsInner(unittest.TestCase):
    """XrayJsonImportExecutionTestsInner unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> XrayJsonImportExecutionTestsInner:
        """Test XrayJsonImportExecutionTestsInner
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `XrayJsonImportExecutionTestsInner`
        """
        model = XrayJsonImportExecutionTestsInner()
        if include_optional:
            return XrayJsonImportExecutionTestsInner(
                comment = '',
                finish = '',
                start = '',
                status = '',
                test_version = '',
                custom_fields = [
                    null
                    ],
                test_info = openapi_client.models.xray_json_import_execution_tests_inner_test_info.XrayJsonImportExecution_tests_inner_testInfo(
                    test_type = '', 
                    project_key = '', 
                    summary = '', 
                    scenario_type = '', 
                    scenario = '', 
                    definition = '', 
                    requirement_keys = '', 
                    test_version_id = 56, ),
                test_key = '',
                executed_by = '',
                evidences = [
                    openapi_client.models.xray_json_import_execution_tests_inner_evidences_inner.XrayJsonImportExecution_tests_inner_evidences_inner(
                        data = '', 
                        filename = '', 
                        content_type = '', )
                    ],
                results = [
                    openapi_client.models.xray_json_import_execution_tests_inner_results_inner.XrayJsonImportExecution_tests_inner_results_inner(
                        name = '', 
                        duration = 56, 
                        log = '', 
                        status = '', 
                        examples = [
                            ''
                            ], )
                    ],
                examples = [
                    ''
                    ],
                steps = [
                    openapi_client.models.xray_json_import_execution_tests_inner_steps_inner.XrayJsonImportExecution_tests_inner_steps_inner(
                        status = '', 
                        comment = '', 
                        evidences = [
                            openapi_client.models.xray_json_import_execution_tests_inner_evidences_inner.XrayJsonImportExecution_tests_inner_evidences_inner(
                                data = '', 
                                filename = '', 
                                content_type = '', )
                            ], 
                        actual_result = '', )
                    ]
            )
        else:
            return XrayJsonImportExecutionTestsInner(
        )
        """

    def testXrayJsonImportExecutionTestsInner(self):
        """Test XrayJsonImportExecutionTestsInner"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.model10_api_test_get200_response_inner import Model10ApiTestGet200ResponseInner

class TestModel10ApiTestGet200ResponseInner(unittest.TestCase):
    """Model10ApiTestGet200ResponseInner unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Model10ApiTestGet200ResponseInner:
        """Test Model10ApiTestGet200ResponseInner
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Model10ApiTestGet200ResponseInner`
        """
        model = Model10ApiTestGet200ResponseInner()
        if include_optional:
            return Model10ApiTestGet200ResponseInner(
                key = '',
                var_self = '',
                reporter = '',
                precondition = [
                    openapi_client.models._1_0_api_test_get_200_response_inner_precondition_inner._1_0_api_test_get_200_response_inner_precondition_inner(
                        precondition_key = '', 
                        self = '', 
                        reporter = '', 
                        assignee = '', 
                        type = '', 
                        condition = '', )
                    ],
                type = '',
                status = '',
                definition = None
            )
        else:
            return Model10ApiTestGet200ResponseInner(
        )
        """

    def testModel10ApiTestGet200ResponseInner(self):
        """Test Model10ApiTestGet200ResponseInner"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from openapi_client.models.defects import Defects
from openapi_client.models.update_iteration_status import UpdateIterationStatus
from openapi_client.models.update_step_result import UpdateStepResult
from openapi_client.models.update_test_run_custom_fields_inner import UpdateTestRunCustomFieldsInner
from openapi_client.models.update_test_run_evidences import UpdateTestRunEvidences
from openapi_client.models.update_test_run_examples import UpdateTestRunExamples
from typing import Optional, Set
from typing_extensions import Self

class UpdateTestRun(BaseModel):
    """
    UpdateTestRun
    """ # noqa: E501
    status: Optional[StrictStr] = None
    comment: Optional[StrictStr] = None
    assignee: Optional[StrictStr] = None
    defects: Optional[Defects] = None
    evidences: Optional[UpdateTestRunEvidences] = None
    custom_fields: Optional[List[UpdateTestRunCustomFieldsInner]] = Field(default=None, alias="customFields")
    iterations: Optional[List[UpdateIterationStatus]] = None
    steps: Optional[List[UpdateStepResult]] = None
    examples: Optional[UpdateTestRunExamples] = None
    __properties: ClassVar[List[str]] = ["status", "comment", "assignee", "defects", "evidences", "customFields", "iterations", "steps", "examples"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UpdateTestRun from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of defects
        if self.defects:
            _dict['defects'] = self.defects.to_dict()
        # override the default output from pydantic by calling `to_dict()` of evidences
        if self.evidences:
            _dict['evidences'] = self.evidences.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in custom_fields (list)
        _items = []
        if self.custom_fields:
            for _item_custom_fields in self.custom_fields:
                if _item_custom_fields:
                    _items.append(_item_custom_fields.to_dict())
            _dict['customFields'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in iterations (list)
        _items = []
        if self.iterations:
            for _item_iterations in self.iterations:
                if _item_iterations:
                    _items.append(_item_iterations.to_dict())
            _dict['iterations'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in steps (list)
        _items = []
        if self.steps:
            for _item_steps in self.steps:
                if _item_steps:
                    _items.append(_item_steps.to_dict())
            _dict['steps'] = _items
        # override the default output from pydantic by calling `to_dict()` of examples
        if self.examples:
            _dict['examples'] = self.examples.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UpdateTestRun from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "status": obj.get("status"),
            "comment": obj.get("comment"),
            "assignee": obj.get("assignee"),
            "defects": Defects.from_dict(obj["defects"]) if obj.get("defects") is not None else None,
            "evidences": UpdateTestRunEvidences.from_dict(obj["evidences"]) if obj.get("evidences") is not None else None,
            "customFields": [UpdateTestRunCustomFieldsInner.from_dict(_item) for _item in obj["customFields"]] if obj.get("customFields") is not None else None,
            "iterations": [UpdateIterationStatus.from_dict(_item) for _item in obj["iterations"]] if obj.get("iterations") is not None else None,
            "steps": [UpdateStepResult.from_dict(_item) for _item in obj["steps"]] if obj.get("steps") is not None else None,
            "examples": UpdateTestRunExamples.from_dict(obj["examples"]) if obj.get("examples") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.test_response_tests_inner import TestResponseTestsInner

class TestTestResponseTestsInner(unittest.TestCase):
    """TestResponseTestsInner unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> TestResponseTestsInner:
        """Test TestResponseTestsInner
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `TestResponseTestsInner`
        """
        model = TestResponseTestsInner()
        if include_optional:
            return TestResponseTestsInner(
                key = '',
                var_self = '',
                reporter = '',
                precondition = {
                    'key' : null
                    },
                type = '',
                status = '',
                definition = ''
            )
        else:
            return TestResponseTestsInner(
        )
        """

    def testTestResponseTestsInner(self):
        """Test TestResponseTestsInner"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.rest_api_success_response_test_issues import RestApiSuccessResponseTestIssues

class TestRestApiSuccessResponseTestIssues(unittest.TestCase):
    """RestApiSuccessResponseTestIssues unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> RestApiSuccessResponseTestIssues:
        """Test RestApiSuccessResponseTestIssues
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `RestApiSuccessResponseTestIssues`
        """
        model = RestApiSuccessResponseTestIssues()
        if include_optional:
            return RestApiSuccessResponseTestIssues(
                success = [
                    openapi_client.models.rest_api_success_response_test_issues_success_inner.RestApiSuccessResponse_testIssues_success_inner(
                        id = '', 
                        key = '', 
                        self = '', 
                        test_version_id = 56, )
                    ],
                error = [
                    openapi_client.models.rest_api_success_response_test_issues_error_inner.RestApiSuccessResponse_testIssues_error_inner(
                        test_definition = '', 
                        messages = [
                            ''
                            ], )
                    ]
            )
        else:
            return RestApiSuccessResponseTestIssues(
        )
        """

    def testRestApiSuccessResponseTestIssues(self):
        """Test RestApiSuccessResponseTestIssues"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# flake8: noqa
"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.called_step import CalledStep
from openapi_client.models.create_step import CreateStep
from openapi_client.models.create_step_request_body import CreateStepRequestBody
from openapi_client.models.create_step_response import CreateStepResponse
from openapi_client.models.create_step_response_step import CreateStepResponseStep
from openapi_client.models.custom_field_meta_information import CustomFieldMetaInformation
from openapi_client.models.data_field import DataField
from openapi_client.models.dataset_parameter import DatasetParameter
from openapi_client.models.defects import Defects
from openapi_client.models.error import Error
from openapi_client.models.field_value import FieldValue
from openapi_client.models.file import File
from openapi_client.models.file_response import FileResponse
from openapi_client.models.folder_rest_bean import FolderRestBean
from openapi_client.models.get_testruns200_response_inner import GetTestruns200ResponseInner
from openapi_client.models.get_testruns200_response_inner_evidences_inner import GetTestruns200ResponseInnerEvidencesInner
from openapi_client.models.get_testruns200_response_inner_steps_inner import GetTestruns200ResponseInnerStepsInner
from openapi_client.models.iteration_step_information import IterationStepInformation
from openapi_client.models.model10_api_testrepository_project_key_folders_folder_id_get200_response import Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response
from openapi_client.models.model10_api_testrun_id_customfield_custom_field_id_put_request import Model10ApiTestrunIdCustomfieldCustomFieldIdPutRequest
from openapi_client.models.model10_api_testrun_id_iteration_iteration_id_step_step_result_id_status_get200_response import Model10ApiTestrunIdIterationIterationIdStepStepResultIdStatusGet200Response
from openapi_client.models.option_custom_field_meta_information import OptionCustomFieldMetaInformation
from openapi_client.models.option_field import OptionField
from openapi_client.models.option_value import OptionValue
from openapi_client.models.project_test_run_cf_meta_information_inner import ProjectTestRunCFMetaInformationInner
from openapi_client.models.project_test_step_cf_meta_information_inner import ProjectTestStepCFMetaInformationInner
from openapi_client.models.rest_api_success_response import RestApiSuccessResponse
from openapi_client.models.rest_api_success_response_test_exec_issue import RestApiSuccessResponseTestExecIssue
from openapi_client.models.rest_api_success_response_test_issues import RestApiSuccessResponseTestIssues
from openapi_client.models.rest_api_success_response_test_issues_error_inner import RestApiSuccessResponseTestIssuesErrorInner
from openapi_client.models.rest_api_success_response_test_issues_success_inner import RestApiSuccessResponseTestIssuesSuccessInner
from openapi_client.models.rest_api_success_response_test_run_warnings_inner import RestApiSuccessResponseTestRunWarningsInner
from openapi_client.models.step import Step
from openapi_client.models.step_custom_field_meta_information import StepCustomFieldMetaInformation
from openapi_client.models.step_fields_value import StepFieldsValue
from openapi_client.models.step_option_custom_field_meta_information import StepOptionCustomFieldMetaInformation
from openapi_client.models.step_response import StepResponse
from openapi_client.models.step_result import StepResult
from openapi_client.models.step_result_actual_result import StepResultActualResult
from openapi_client.models.step_result_comment import StepResultComment
from openapi_client.models.step_results_response import StepResultsResponse
from openapi_client.models.test_example_step_bean import TestExampleStepBean
from openapi_client.models.test_response_inner import TestResponseInner
from openapi_client.models.test_response_inner_definition import TestResponseInnerDefinition
from openapi_client.models.test_response_inner_definition_one_of import TestResponseInnerDefinitionOneOf
from openapi_client.models.test_response_inner_definition_one_of_steps_inner import TestResponseInnerDefinitionOneOfStepsInner
from openapi_client.models.test_response_inner_definition_one_of_steps_inner_step import TestResponseInnerDefinitionOneOfStepsInnerStep
from openapi_client.models.test_response_inner_precondition_inner import TestResponseInnerPreconditionInner
from openapi_client.models.test_run_custom_field_multi import TestRunCustomFieldMulti
from openapi_client.models.test_run_custom_field_response import TestRunCustomFieldResponse
from openapi_client.models.test_run_custom_field_single import TestRunCustomFieldSingle
from openapi_client.models.test_run_example import TestRunExample
from openapi_client.models.test_run_iteration_information import TestRunIterationInformation
from openapi_client.models.test_run_iteration_status_information import TestRunIterationStatusInformation
from openapi_client.models.test_run_response import TestRunResponse
from openapi_client.models.test_run_response_custom_fields_inner import TestRunResponseCustomFieldsInner
from openapi_client.models.test_step_result_evidence_input import TestStepResultEvidenceInput
from openapi_client.models.test_steps_response import TestStepsResponse
from openapi_client.models.test_steps_response_steps_inner import TestStepsResponseStepsInner
from openapi_client.models.update_iteration import UpdateIteration
from openapi_client.models.update_iteration_response import UpdateIterationResponse
from openapi_client.models.update_iteration_status import UpdateIterationStatus
from openapi_client.models.update_step import UpdateStep
from openapi_client.models.update_step_attachments import UpdateStepAttachments
from openapi_client.models.update_step_request_body import UpdateStepRequestBody
from openapi_client.models.update_step_result import UpdateStepResult
from openapi_client.models.update_step_result_defects import UpdateStepResultDefects
from openapi_client.models.update_step_result_evidences import UpdateStepResultEvidences
from openapi_client.models.update_tr_custom_field_value_multi import UpdateTRCustomFieldValueMulti
from openapi_client.models.update_tr_custom_field_value_single import UpdateTRCustomFieldValueSingle
from openapi_client.models.update_test_run import UpdateTestRun
from openapi_client.models.update_test_run_custom_field_multi import UpdateTestRunCustomFieldMulti
from openapi_client.models.update_test_run_custom_field_single import UpdateTestRunCustomFieldSingle
from openapi_client.models.update_test_run_custom_fields_inner import UpdateTestRunCustomFieldsInner
from openapi_client.models.update_test_run_evidences import UpdateTestRunEvidences
from openapi_client.models.update_test_run_examples import UpdateTestRunExamples
from openapi_client.models.update_test_run_response import UpdateTestRunResponse
from openapi_client.models.version import Version
from openapi_client.models.wiki_field import WikiField
from openapi_client.models.wiki_value import WikiValue
from openapi_client.models.xray_json_import_execution import XrayJsonImportExecution
from openapi_client.models.xray_json_import_execution_info import XrayJsonImportExecutionInfo
from openapi_client.models.xray_json_import_execution_tests_inner import XrayJsonImportExecutionTestsInner
from openapi_client.models.xray_json_import_execution_tests_inner_evidences_inner import XrayJsonImportExecutionTestsInnerEvidencesInner
from openapi_client.models.xray_json_import_execution_tests_inner_results_inner import XrayJsonImportExecutionTestsInnerResultsInner
from openapi_client.models.xray_json_import_execution_tests_inner_steps_inner import XrayJsonImportExecutionTestsInnerStepsInner
from openapi_client.models.xray_json_import_execution_tests_inner_test_info import XrayJsonImportExecutionTestsInnerTestInfo
from openapi_client.models.xray_license_status import XrayLicenseStatus



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictInt, StrictStr
from typing import Optional
from typing_extensions import Annotated
from openapi_client.models.model10_api_testrepository_project_key_folders_folder_id_get200_response import Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response

from openapi_client.api_client import ApiClient, RequestSerialized
from openapi_client.api_response import ApiResponse
from openapi_client.rest import RESTResponseType


class TestRepositoryApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_delete(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """call_10_api_testrepository_project_key_folders_folder_id_delete

        Creates a folder and returns a JSON object with folder information.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_delete_serialize(
            project_key=project_key,
            folder_id=folder_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': "Error",
            '401': "Error",
            '500': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_delete_with_http_info(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """call_10_api_testrepository_project_key_folders_folder_id_delete

        Creates a folder and returns a JSON object with folder information.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_delete_serialize(
            project_key=project_key,
            folder_id=folder_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': "Error",
            '401': "Error",
            '500': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_delete_without_preload_content(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """call_10_api_testrepository_project_key_folders_folder_id_delete

        Creates a folder and returns a JSON object with folder information.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_delete_serialize(
            project_key=project_key,
            folder_id=folder_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': "Error",
            '401': "Error",
            '500': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _call_10_api_testrepository_project_key_folders_folder_id_delete_serialize(
        self,
        project_key,
        folder_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if project_key is not None:
            _path_params['projectKey'] = project_key
        if folder_id is not None:
            _path_params['folderId'] = folder_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BasicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/1.0/api/testrepository/{projectKey}/folders/{folderId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_get(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response:
        """call_10_api_testrepository_project_key_folders_folder_id_get

        Retrieves information about a given Test Repository folder. <br/> The response will contain all information related to that folder, e.g., id, name, tests count and direct children folder ids. <br/>

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_get_serialize(
            project_key=project_key,
            folder_id=folder_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_get_with_http_info(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response]:
        """call_10_api_testrepository_project_key_folders_folder_id_get

        Retrieves information about a given Test Repository folder. <br/> The response will contain all information related to that folder, e.g., id, name, tests count and direct children folder ids. <br/>

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_get_serialize(
            project_key=project_key,
            folder_id=folder_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_get_without_preload_content(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """call_10_api_testrepository_project_key_folders_folder_id_get

        Retrieves information about a given Test Repository folder. <br/> The response will contain all information related to that folder, e.g., id, name, tests count and direct children folder ids. <br/>

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_get_serialize(
            project_key=project_key,
            folder_id=folder_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _call_10_api_testrepository_project_key_folders_folder_id_get_serialize(
        self,
        project_key,
        folder_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if project_key is not None:
            _path_params['projectKey'] = project_key
        if folder_id is not None:
            _path_params['folderId'] = folder_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BasicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/1.0/api/testrepository/{projectKey}/folders/{folderId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_post(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        name: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response:
        """call_10_api_testrepository_project_key_folders_folder_id_post

        Creates a folder and returns a JSON object with folder information.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param name: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type name: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_post_serialize(
            project_key=project_key,
            folder_id=folder_id,
            name=name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_post_with_http_info(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        name: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response]:
        """call_10_api_testrepository_project_key_folders_folder_id_post

        Creates a folder and returns a JSON object with folder information.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param name: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type name: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_post_serialize(
            project_key=project_key,
            folder_id=folder_id,
            name=name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_post_without_preload_content(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        name: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """call_10_api_testrepository_project_key_folders_folder_id_post

        Creates a folder and returns a JSON object with folder information.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param name: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type name: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_post_serialize(
            project_key=project_key,
            folder_id=folder_id,
            name=name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _call_10_api_testrepository_project_key_folders_folder_id_post_serialize(
        self,
        project_key,
        folder_id,
        name,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if project_key is not None:
            _path_params['projectKey'] = project_key
        if folder_id is not None:
            _path_params['folderId'] = folder_id
        # process the query parameters
        if name is not None:
            
            _query_params.append(('name', name))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BasicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/1.0/api/testrepository/{projectKey}/folders/{folderId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_put(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        name: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response:
        """call_10_api_testrepository_project_key_folders_folder_id_put

        Creates a folder and returns a JSON object with folder information.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param name: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type name: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_put_serialize(
            project_key=project_key,
            folder_id=folder_id,
            name=name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_put_with_http_info(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        name: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response]:
        """call_10_api_testrepository_project_key_folders_folder_id_put

        Creates a folder and returns a JSON object with folder information.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param name: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type name: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_put_serialize(
            project_key=project_key,
            folder_id=folder_id,
            name=name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_put_without_preload_content(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        name: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """call_10_api_testrepository_project_key_folders_folder_id_put

        Creates a folder and returns a JSON object with folder information.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param name: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type name: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_put_serialize(
            project_key=project_key,
            folder_id=folder_id,
            name=name,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _call_10_api_testrepository_project_key_folders_folder_id_put_serialize(
        self,
        project_key,
        folder_id,
        name,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if project_key is not None:
            _path_params['projectKey'] = project_key
        if folder_id is not None:
            _path_params['folderId'] = folder_id
        # process the query parameters
        if name is not None:
            
            _query_params.append(('name', name))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BasicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/1.0/api/testrepository/{projectKey}/folders/{folderId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_tests_get(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        all_descendants: Annotated[Optional[StrictBool], Field(description="include all descendants (i.e. all child Tests); \"false\", by default Repository")] = None,
        page: Annotated[Optional[StrictInt], Field(description="page of paginated data (first 1)")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="amount of Tests per paginated data")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response:
        """call_10_api_testrepository_project_key_folders_folder_id_tests_get

        Returns a JSON object with a list of the Tests contained in a given folder of the Test Repository. This endpoint supports pagination.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param all_descendants: include all descendants (i.e. all child Tests); \"false\", by default Repository
        :type all_descendants: bool
        :param page: page of paginated data (first 1)
        :type page: int
        :param limit: amount of Tests per paginated data
        :type limit: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_tests_get_serialize(
            project_key=project_key,
            folder_id=folder_id,
            all_descendants=all_descendants,
            page=page,
            limit=limit,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_tests_get_with_http_info(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        all_descendants: Annotated[Optional[StrictBool], Field(description="include all descendants (i.e. all child Tests); \"false\", by default Repository")] = None,
        page: Annotated[Optional[StrictInt], Field(description="page of paginated data (first 1)")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="amount of Tests per paginated data")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response]:
        """call_10_api_testrepository_project_key_folders_folder_id_tests_get

        Returns a JSON object with a list of the Tests contained in a given folder of the Test Repository. This endpoint supports pagination.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param all_descendants: include all descendants (i.e. all child Tests); \"false\", by default Repository
        :type all_descendants: bool
        :param page: page of paginated data (first 1)
        :type page: int
        :param limit: amount of Tests per paginated data
        :type limit: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_tests_get_serialize(
            project_key=project_key,
            folder_id=folder_id,
            all_descendants=all_descendants,
            page=page,
            limit=limit,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_tests_get_without_preload_content(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        all_descendants: Annotated[Optional[StrictBool], Field(description="include all descendants (i.e. all child Tests); \"false\", by default Repository")] = None,
        page: Annotated[Optional[StrictInt], Field(description="page of paginated data (first 1)")] = None,
        limit: Annotated[Optional[StrictInt], Field(description="amount of Tests per paginated data")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """call_10_api_testrepository_project_key_folders_folder_id_tests_get

        Returns a JSON object with a list of the Tests contained in a given folder of the Test Repository. This endpoint supports pagination.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param all_descendants: include all descendants (i.e. all child Tests); \"false\", by default Repository
        :type all_descendants: bool
        :param page: page of paginated data (first 1)
        :type page: int
        :param limit: amount of Tests per paginated data
        :type limit: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_tests_get_serialize(
            project_key=project_key,
            folder_id=folder_id,
            all_descendants=all_descendants,
            page=page,
            limit=limit,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _call_10_api_testrepository_project_key_folders_folder_id_tests_get_serialize(
        self,
        project_key,
        folder_id,
        all_descendants,
        page,
        limit,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if project_key is not None:
            _path_params['projectKey'] = project_key
        if folder_id is not None:
            _path_params['folderId'] = folder_id
        # process the query parameters
        if all_descendants is not None:
            
            _query_params.append(('allDescendants', all_descendants))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BasicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/1.0/api/testrepository/{projectKey}/folders/{folderId}/tests',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_tests_put(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response:
        """call_10_api_testrepository_project_key_folders_folder_id_tests_put

        Returns a JSON object with a list of the Tests contained in a given folder of the Test Repository. This endpoint supports pagination.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_tests_put_serialize(
            project_key=project_key,
            folder_id=folder_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_tests_put_with_http_info(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response]:
        """call_10_api_testrepository_project_key_folders_folder_id_tests_put

        Returns a JSON object with a list of the Tests contained in a given folder of the Test Repository. This endpoint supports pagination.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_tests_put_serialize(
            project_key=project_key,
            folder_id=folder_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def call_10_api_testrepository_project_key_folders_folder_id_tests_put_without_preload_content(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        folder_id: Annotated[StrictInt, Field(description="The folder id.\"-1\" corresponds to the root folder of the Test Repository")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """call_10_api_testrepository_project_key_folders_folder_id_tests_put

        Returns a JSON object with a list of the Tests contained in a given folder of the Test Repository. This endpoint supports pagination.

        :param project_key: The project key (required)
        :type project_key: str
        :param folder_id: The folder id.\"-1\" corresponds to the root folder of the Test Repository (required)
        :type folder_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_folder_id_tests_put_serialize(
            project_key=project_key,
            folder_id=folder_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "Model10ApiTestrepositoryProjectKeyFoldersFolderIdGet200Response",
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _call_10_api_testrepository_project_key_folders_folder_id_tests_put_serialize(
        self,
        project_key,
        folder_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if project_key is not None:
            _path_params['projectKey'] = project_key
        if folder_id is not None:
            _path_params['folderId'] = folder_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BasicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/1.0/api/testrepository/{projectKey}/folders/{folderId}/tests',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def call_10_api_testrepository_project_key_folders_get(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """call_10_api_testrepository_project_key_folders_get

        Returns a JSON object with a list of the folders of the Test Repository.

        :param project_key: The project key (required)
        :type project_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_get_serialize(
            project_key=project_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def call_10_api_testrepository_project_key_folders_get_with_http_info(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """call_10_api_testrepository_project_key_folders_get

        Returns a JSON object with a list of the folders of the Test Repository.

        :param project_key: The project key (required)
        :type project_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_get_serialize(
            project_key=project_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def call_10_api_testrepository_project_key_folders_get_without_preload_content(
        self,
        project_key: Annotated[StrictStr, Field(description="The project key")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """call_10_api_testrepository_project_key_folders_get

        Returns a JSON object with a list of the folders of the Test Repository.

        :param project_key: The project key (required)
        :type project_key: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._call_10_api_testrepository_project_key_folders_get_serialize(
            project_key=project_key,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _call_10_api_testrepository_project_key_folders_get_serialize(
        self,
        project_key,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if project_key is not None:
            _path_params['projectKey'] = project_key
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BasicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/1.0/api/testrepository/{projectKey}/folders',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.test_run_custom_field_single import TestRunCustomFieldSingle

class TestTestRunCustomFieldSingle(unittest.TestCase):
    """TestRunCustomFieldSingle unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> TestRunCustomFieldSingle:
        """Test TestRunCustomFieldSingle
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `TestRunCustomFieldSingle`
        """
        model = TestRunCustomFieldSingle()
        if include_optional:
            return TestRunCustomFieldSingle(
                id = 28,
                name = 'Toogle Custom Field',
                value = 'true'
            )
        else:
            return TestRunCustomFieldSingle(
        )
        """

    def testTestRunCustomFieldSingle(self):
        """Test TestRunCustomFieldSingle"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.update_step import UpdateStep

class TestUpdateStep(unittest.TestCase):
    """UpdateStep unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> UpdateStep:
        """Test UpdateStep
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `UpdateStep`
        """
        model = UpdateStep()
        if include_optional:
            return UpdateStep(
                fields = {"Action":"Step Action","Date":"02/Feb/2020","Toggle":"true","SingleSelectList":"Selected Value","MultiSelectList":["Option A","Option B"]},
                attachments = openapi_client.models.update_step_attachments.UpdateStep_attachments(
                    add = [
                        openapi_client.models.file.File(
                            data = 'gsddfgdsfg...(base64)', 
                            filename = 'example.txt', 
                            content_type = 'plain/text', )
                        ], 
                    remove = [1234,4546], )
            )
        else:
            return UpdateStep(
        )
        """

    def testUpdateStep(self):
        """Test UpdateStep"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.step_result import StepResult

class TestStepResult(unittest.TestCase):
    """StepResult unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> StepResult:
        """Test StepResult
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `StepResult`
        """
        model = StepResult()
        if include_optional:
            return StepResult(
                id = 85778,
                index = 1,
                fields = {"Action":{"type":"Wiki","value":{"raw":"This is a step action","rendered":"<p>This is a step action</p>"}},"Toggle":{"type":"Data","value":"false"}},
                attachments = [
                    {"id":25684,"fileName":"attachment.txt","fileIcon":"text.gif","mimeType":"plain/text","fileIconAlt":"Text File","fileSize":"100.0 kB","numericalFileSize":"12","created":"2020-02-06T10:39:37Z","createdDate":1580985577000,"author":"admin","authorFullName":"Bruce Wayne","fileURL":"http://example.com/attachment/25684/example2.txt","filePath":"/Users/example/path/attachment"}
                    ],
                status = 'PASS',
                comment = openapi_client.models.step_result_comment.StepResult_comment(
                    rendered = 'Comment on step', ),
                defects = ["CALC-123","CALC-765"],
                evidences = [
                    {"id":25684,"fileName":"attachment.txt","fileIcon":"text.gif","mimeType":"plain/text","fileIconAlt":"Text File","fileSize":"100.0 kB","numericalFileSize":"12","created":"2020-02-06T10:39:37Z","createdDate":1580985577000,"author":"admin","authorFullName":"Bruce Wayne","fileURL":"http://example.com/attachment/25684/example2.txt","filePath":"/Users/example/path/attachment"}
                    ],
                actual_result = openapi_client.models.step_result_actual_result.StepResult_actualResult(
                    rendered = 'Actual result on step', )
            )
        else:
            return StepResult(
        )
        """

    def testStepResult(self):
        """Test StepResult"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.update_test_run_response import UpdateTestRunResponse

class TestUpdateTestRunResponse(unittest.TestCase):
    """UpdateTestRunResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> UpdateTestRunResponse:
        """Test UpdateTestRunResponse
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `UpdateTestRunResponse`
        """
        model = UpdateTestRunResponse()
        if include_optional:
            return UpdateTestRunResponse(
                id = 1004725,
                evidence_ids = [
                    56
                    ],
                warnings = [
                    '["Test example with id 1379 does not belong to test run with id 1004725."]'
                    ],
                step_results = [
                    openapi_client.models.step_results_response.StepResultsResponse(
                        id = 813743, 
                        evidence_ids = [
                            56
                            ], 
                        warnings = [
                            '["No attachment with id 123 was found."]'
                            ], )
                    ]
            )
        else:
            return UpdateTestRunResponse(
        )
        """

    def testUpdateTestRunResponse(self):
        """Test UpdateTestRunResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# flake8: noqa

# import apis into api package
from openapi_client.api.dataset_api import DatasetApi
from openapi_client.api.external_apps_api import ExternalAppsApi
from openapi_client.api.import_api import ImportApi
from openapi_client.api.pre_conditions_api import PreConditionsApi
from openapi_client.api.project_api import ProjectApi
from openapi_client.api.requirement_status_api import RequirementStatusApi
from openapi_client.api.test_execution_tests_api import TestExecutionTestsApi
from openapi_client.api.test_plan_test_executions_api import TestPlanTestExecutionsApi
from openapi_client.api.test_plan_tests_api import TestPlanTestsApi
from openapi_client.api.test_repository_api import TestRepositoryApi
from openapi_client.api.test_run_api import TestRunApi
from openapi_client.api.test_run_status_api import TestRunStatusApi
from openapi_client.api.test_sets_api import TestSetsApi
from openapi_client.api.test_step_api import TestStepApi
from openapi_client.api.tests_api import TestsApi
from openapi_client.api.jira_functions import JiraFunctions



# coding: utf-8

"""
    Xray REST API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 2.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.models.xray_json_import_execution import XrayJsonImportExecution

class TestXrayJsonImportExecution(unittest.TestCase):
    """XrayJsonImportExecution unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> XrayJsonImportExecution:
        """Test XrayJsonImportExecution
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `XrayJsonImportExecution`
        """
        model = XrayJsonImportExecution()
        if include_optional:
            return XrayJsonImportExecution(
                test_execution_key = '',
                info = openapi_client.models.xray_json_import_execution_info.XrayJsonImportExecution_info(
                    project = '', 
                    summary = '', 
                    description = '', 
                    user = '', 
                    version = '', 
                    revision = '', 
                    start_date = '', 
                    finish_date = '', ),
                tests = [
                    openapi_client.models.xray_json_import_execution_tests_inner.XrayJsonImportExecution_tests_inner(
                        comment = '', 
                        finish = '', 
                        start = '', 
                        status = '', 
                        test_version = '', 
                        custom_fields = [
                            null
                            ], 
                        test_info = openapi_client.models.xray_json_import_execution_tests_inner_test_info.XrayJsonImportExecution_tests_inner_testInfo(
                            test_type = '', 
                            project_key = '', 
                            summary = '', 
                            scenario_type = '', 
                            scenario = '', 
                            definition = '', 
                            requirement_keys = '', 
                            test_version_id = 56, ), 
                        test_key = '', 
                        executed_by = '', 
                        evidences = [
                            openapi_client.models.xray_json_import_execution_tests_inner_evidences_inner.XrayJsonImportExecution_tests_inner_evidences_inner(
                                data = '', 
                                filename = '', 
                                content_type = '', )
                            ], 
                        results = [
                            openapi_client.models.xray_json_import_execution_tests_inner_results_inner.XrayJsonImportExecution_tests_inner_results_inner(
                                name = '', 
                                duration = 56, 
                                log = '', 
                                status = '', 
                                examples = [
                                    ''
                                    ], )
                            ], 
                        examples = [
                            ''
                            ], 
                        steps = [
                            openapi_client.models.xray_json_import_execution_tests_inner_steps_inner.XrayJsonImportExecution_tests_inner_steps_inner(
                                status = '', 
                                comment = '', 
                                actual_result = '', )
                            ], )
                    ]
            )
        else:
            return XrayJsonImportExecution(
        )
        """

    def testXrayJsonImportExecution(self):
        """Test XrayJsonImportExecution"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



import pickle
import os
import sys
import random
import time
from inspect import currentframe, getframeinfo
import requests
import yaml

from jira import JIRA, JIRAError
from requests import ReadTimeout
from yaml import YAMLError
from openapi_client.api_client import ApiClient
from openapi_client.api.tests_api import TestsApi

class JiraFunctions:

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    def preprocess_url(self, url):
        """Preprocess URL to ensure format is correct for API."""
        if url[-1] == '/':
            return url[:-1]
        return url


    def server_auth(self, raw_url, pa_token, retry_count=-1):
        """Perform server setup and check for successful login."""
        retry_count_req = os.getenv('JIRA_RETRY_COUNT')
        if retry_count_req == '' or retry_count_req is None:
            retry_count_req = 10
        else:
            retry_count_req = int(retry_count_req)
        retry_count_req = os.getenv('JIRA_RETRY_COUNT')
        if retry_count_req == '' or retry_count_req is None:
            retry_count_req = 10
        else:
            retry_count_req = int(retry_count_req)
        url = self.preprocess_url(raw_url)
        # connect to server
        server = JIRA(server=url, token_auth=pa_token)
        try:
            server.myself()
        except JIRAError as ex:
            response = ex.response.headers
            status_code = ex.status_code
            if status_code == 429:
                if retry_count == -1:
                    retry_count_inc = 0
                else:
                    retry_count_inc = retry_count
                backoff_time = int(response['retry-after']) + \
                    random.randint(60, 80) + (retry_count_inc * 5)
                if retry_count < retry_count_req:
                    time.sleep(backoff_time)
                    retry_count += 1
                    self.server_auth(raw_url, pa_token, retry_count)
                else:
                    print('Error raised from line: ' +
                        str(getframeinfo(currentframe()).lineno))
                    raise JIRAError from ex

            else:
                print(
                    'Error! Could not authenticate with JIRA. Ensure your JIRA_PA_TOKEN is correct.')
                sys.exit(1)
        return server, url


    def check_xray_file(self):
        """Function to return filename of XRAY Configuration File."""
        projpath = os.environ['CI_PROJECT_PATH']
        runnerfilepath_yml = os.path.join('/builds', projpath, 'xray_filepath.yml')
        runnerfilepath_yaml = os.path.join(
            '/builds', projpath, 'xray_filepath.yaml')
        if os.path.exists(runnerfilepath_yml):
            file_name = runnerfilepath_yml
        elif os.path.exists(runnerfilepath_yaml):
            file_name = runnerfilepath_yaml
        else:
            raise FileNotFoundError('Could not find xray_filepath.yaml or' +
                                    ' xray_filepath.yaml in your project root!')
        return file_name


    def get_yaml_value(self, model_name, yaml_key):
        """Function to get value from a YAML file, by specified key"""
        xray_file_name = self.check_xray_file()
        if xray_file_name != '':
            # Load XRAY File
            with open(xray_file_name, 'r', encoding='UTF-8') as xray_file_id:
                xray_file = yaml.safe_load(xray_file_id)
            # Try to get YAML key for specified model name
            xray_path = xray_file.get(model_name)
            try:
                # If the model name is not specified in the YAML
                if xray_path is None:
                    # Look for the default key and use the value from there
                    xray_path = xray_file.get('default')
                    pid_val = xray_path.get(yaml_key)
                # If the model name is specified, return PID ticket value if
                # present
                else:
                    pid_val = xray_path.get(yaml_key)
                    if pid_val is None:
                        default_section = xray_file.get('default')
                        pid_val = default_section.get(yaml_key)
                return pid_val
            except YAMLError as e:
                print('Failed to find ' + yaml_key + ' in XRAY File!')
                raise e


    def get_pid(self, model_name):
        """Wrapper function to get PID Ticket # from XRAY Config File"""

        return self.get_yaml_value(model_name, 'PID Ticket')


    def get_model_xray_repo_path(self, model_name):
        """Wrapper function to get XRAY Test Repository Path from XRAY Config File"""
        return self.get_yaml_value(model_name, 'Test Repository Path')


    def get_retry_count(self) -> int:
        """Returns the value of JIRA_RETRY_COUNT, or a default value (10) if none given"""
        retry_count_req = os.getenv('JIRA_RETRY_COUNT')
        if retry_count_req == '' or retry_count_req is None:
            retry_count_req = 10
        else:
            retry_count_req = int(retry_count_req)

        return retry_count_req


    def common_backoff_handler(self, ex, retry_count):
        """Function to handle backoff from calls to the JIRA API."""
        retry_count_req = self.get_retry_count()
        status_code = ex.status_code
        response = ex.response.headers
        if status_code == 429:
            if retry_count == -1:
                retry_count_inc = 0
            else:
                retry_count_inc = retry_count
            backoff_time = int(response['retry-after']) + \
                random.randint(60, 80) + (retry_count_inc * 5)
            if retry_count < retry_count_req:
                time.sleep(backoff_time)
                retry_count += 1
                return retry_count
            print('Exit Code: ' + str(status_code))
            print('Response Headers: ' + str(response))
            print('Response Text: ' + str(ex.text))
            return -2
        if str(status_code).startswith('2'):
            return 0
        print('Exit Code: ' + str(status_code))
        print('Response Headers: ' + str(response))
        print('Response Text: ' + str(ex.text))
        return -2


    def get_post_put_backoff_handler(self, response, retry_count):
        """Function for POST/PUT requests to handle rate limit backoffs."""
        response_headers = response.headers
        retry_count_req = self.get_retry_count()
        if str(response.status_code).startswith('2'):
            return -3
        if response.status_code == 429:
            if retry_count == -1:
                retry_count_inc = 0
            else:
                retry_count_inc = retry_count
            backoff_time = int(
                response_headers['retry-after']) + random.randint(60, 80) + (retry_count_inc * 5)
            if retry_count < retry_count_req:
                time.sleep(backoff_time)
                retry_count += 1
                return retry_count
            print('ERROR: Rate Limit Exceeded for Adding Test to Test Plan!')
            return -2


    def get_issue_handle_ratelim(self, server, issue_id, retry_count=-1):
        """Function to get an issue from JIRA, handling rate limits."""
        try:
            issu = server.issue(issue_id)
            print('Retrieved issue: ' + issue_id)
            return issu
        except JIRAError as ex:
            retry_count = self.common_backoff_handler(ex, retry_count)
            if retry_count > -2:
                self.get_issue_handle_ratelim(server, issue_id, retry_count)
            else:
                print('ERROR: Rate Limit Exceeded for Issue Collection!')
                print('Error raised from line: ' +
                    str(getframeinfo(currentframe()).lineno))
                raise JIRAError from ex


    def create_issue_handle_ratelim(self, server, target_dict, retry_count=-1):
        """Function to create an issue using a dictionary, handling rate limits."""

        try:
            issue = TestsApi.call_10_api_settings_teststatuses_get(fields=target_dict)
            return issue
        except JIRAError as ex:
            retry_count = self.common_backoff_handler(ex, retry_count)
            if retry_count > -2:
                self.create_issue_handle_ratelim(server, target_dict, retry_count)
            else:
                print('ERROR: Rate Limit Exceeded for Issue Creation!')
                print('Error raised from line: ' +
                    str(getframeinfo(currentframe()).lineno))
                raise JIRAError from ex


    def transition_issue_handle_ratelim(
            self,
            server,
            issue_object,
            transition_id,
            retry_count=-1,
            comment=''):
        """Function to transition issue using an object, handling rate limits"""
        try:
            server.transition_issue(issue_object, transition_id, comment=comment)
        except JIRAError as ex:
            retry_count = self.common_backoff_handler(ex, retry_count)
            if retry_count > -2:
                self.transition_issue_handle_ratelim(
                    server, issue_object, transition_id, retry_count, comment)
            else:
                print('ERROR: Rate Limit Exceeded for Transitions!')
                print('Error raised from line: ' +
                    str(getframeinfo(currentframe()).lineno))
                raise JIRAError from ex


    def update_issue_handle_ratelim(self,issue_obj, update_dict, retry_count=-1):
        """Function to update an issue with a specified field name and content, handling rate limits."""
        try:
            issue_obj.update(fields=update_dict)
        except JIRAError as ex:
            retry_count = self.common_backoff_handler(ex, retry_count)
            if retry_count > -2:
                self.update_issue_handle_ratelim(issue_obj, update_dict, retry_count)
            else:
                print('ERROR: Rate Limit Exceeded for Issue Update!')
                print('Error raised from line: ' +
                    str(getframeinfo(currentframe()).lineno))
                raise JIRAError from ex


    def add_field_value_handle_ratelim(
            self,
            issue_obj,
            field_name,
            field_value,
            retry_count=-1):
        """Function to update field value, handling rate limits."""
        try:
            issue_obj.add_field_value(field_name, field_value)
        except JIRAError as ex:
            retry_count = self.common_backoff_handler(ex, retry_count)
            if retry_count > -2:
                self.add_field_value_handle_ratelim(
                    issue_obj, field_name, field_value, retry_count)
            else:
                print('ERROR: Rate Limit Exceeded for Adding Field Value!')
                print('Error raised from line: ' +
                    str(getframeinfo(currentframe()).lineno))
                raise JIRAError from ex


    def get_custom_field_option(self, server, option_id, retry_count=-1):
        """Function to retrieve custom field from server, handle backoff and
        perform local pickling of server response for later reuse."""
        pickle_filename = option_id + '_jiracustomfield.pkl'
        if os.path.exists(pickle_filename):
            with open(pickle_filename, 'rb') as pickle_file:
                # nosemgrep
                return pickle.load(pickle_file)
        else:
            try:
                custom_field = server.custom_field_option(option_id)
            except JIRAError as ex:
                retry_count = self.common_backoff_handler(ex, retry_count)
                if retry_count > -2:
                    self.get_custom_field_option(server, option_id, retry_count)
                else:
                    print('ERROR: Rate Limit Exceeded for Get Custom Field Option!')
                    print('Error raised from line: ' +
                        str(getframeinfo(currentframe()).lineno))
                    raise JIRAError from ex
            # pickle custom field for later retrieval
            with open(pickle_filename, 'wb') as pickle_file:
                # nosemgrep
                pickle.dump(custom_field, pickle_file)
            return custom_field


    def post_request_handle_ratelim(
            self,
            request_url,
            request_headers,
            request_json,
            retry_count=-1):
        """Function to perform POST request with rate limit handling."""
        try:
            response = requests.post(
                request_url,
                json=request_json,
                headers=request_headers,
                timeout=60)
            retry_count = -2
        except ReadTimeout:
            retry_count = self.get_post_put_backoff_handler(response, retry_count)
        if retry_count > -2:  # Keep retrying
            self.post_request_handle_ratelim(
                request_url, request_headers, request_json, retry_count)
        else:
            return response.status_code

    def validate_url(self, url):
        if url.endswith(('.jlr-apps.com', '.jlrmotor.com')):
            return True
        else:
            return False

    def put_request_handle_ratelim(self, request_url,request_headers,retry_count=-1):
        """Function to perform PUT request with rate limit handling."""
        if self.validate_url(request_url):
            try:
                response = requests.put(request_url,headers=request_headers,timeout=60)
                retry_count = -2
            except ReadTimeout:
                retry_count = self.get_post_put_backoff_handler(response,retry_count)
            if retry_count > -2: #Keep retrying
                self.put_request_handle_ratelim(request_url,request_headers,retry_count)
            else:
                return 'Error: Expected URL to end in .jlr-apps.com or .jlrmotor.com'

    def put_request_handle_ratelim(self, request_url, request_headers, retry_count=-1):
        """Function to perform PUT request with rate limit handling."""
        try:
            response = requests.put(
                request_url, headers=request_headers, timeout=60)
            retry_count = -2
        except ReadTimeout:
            retry_count = self.get_post_put_backoff_handler(response, retry_count)
        if retry_count > -2:  # Keep retrying
            self.put_request_handle_ratelim(request_url, request_headers, retry_count)
        else:
            return 'Error: Expected URL to end in .jlr-apps.com or .jlrmotor.com'


    def get_request_handle_ratelim(self, request_url, request_headers, retry_count=-1):
        """Function to perform GET requests whilst handling rate limiting."""
        try:
            response = requests.get(
                request_url, headers=request_headers, timeout=60)
            retry_count = -2
        except ReadTimeout:
            retry_count = self.get_post_put_backoff_handler(response, retry_count)
        if retry_count > -2:  # Keep retrying
            self.get_request_handle_ratelim(request_url, request_headers, retry_count)
        else:
            return response


    def transition_to_approved(self, server, testcase_obj):
        """Function to transition to Approved state based on current state."""
        testcase_status = testcase_obj.fields.status.name
        if testcase_status == 'In Progress':
            # Transition Issue to Under Review State
            self.transition_issue_handle_ratelim(server, testcase_obj, '21')
            # Transition Issue to Approved State
            self.transition_issue_handle_ratelim(server, testcase_obj, '101')
        elif testcase_status == 'Under Review':
            # Transition Issue to Approved State
            self.transition_issue_handle_ratelim(server, testcase_obj, '101')
        elif testcase_status == 'Cancelled':
            # Transition Issue to New State
            self.transition_issue_handle_ratelim(server, testcase_obj, '41')
            # Transition to 'In Progress'
            self.transition_issue_handle_ratelim(server, testcase_obj, '91')
            # Transition Issue to Under Review State
            self.transition_issue_handle_ratelim(server, testcase_obj, '21')
            # Transition Issue to Approved State
            self.transition_issue_handle_ratelim(server, testcase_obj, '101')
        elif testcase_status == 'Rejected':
            # Transition to 'In Progress'
            self.transition_issue_handle_ratelim(server, testcase_obj, '81')
            # Transition Issue to Under Review State
            self.transition_issue_handle_ratelim(server, testcase_obj, '21')
            # Transition Issue to Approved State
            self.transition_issue_handle_ratelim(server, testcase_obj, '101')
        elif testcase_status == 'New':
            # Transition to 'In Progress'
            self.transition_issue_handle_ratelim(server, testcase_obj, '91')
            # Transition Issue to Under Review State
            self.transition_issue_handle_ratelim(server, testcase_obj, '21')
            # Transition Issue to Approved State
            self.transition_issue_handle_ratelim(server, testcase_obj, '101')


    def transition_to_in_progress(self, server, testcase_obj):
        """Function to transition to In Progress state based on current state."""
        testcase_status = testcase_obj.fields.status.name
        if testcase_status == 'New':
            # Transition to 'In Progress'
            self.transition_issue_handle_ratelim(server, testcase_obj, '91')
        elif testcase_status == 'Under Review':
            # Transition to 'Rejected'
            self.transition_issue_handle_ratelim(
                server,
                testcase_obj,
                '51',
                comment='Rejected as part of Transition to In Progress')
            # Transition to 'In Progress'
            self.transition_issue_handle_ratelim(server, testcase_obj, '81')
        elif testcase_status == 'Rejected':
            # Transition to 'In Progress'
            self.transition_issue_handle_ratelim(server, testcase_obj, '81')
        elif testcase_status == 'Cancelled':
            # Transition to 'New'
            self.transition_issue_handle_ratelim(server, testcase_obj, '41')
            # Transition to 'In Progress'
            self.transition_issue_handle_ratelim(server, testcase_obj, '91')
        elif testcase_status == 'Approved':
            # Transition to 'In Progress'
            self.transition_issue_handle_ratelim(server, testcase_obj, '81')


    def create_test_set(self, url, pa_token, project_key, ts_name):
        """Function to create XRAY Test Set for Simulink Test Suite"""
        server, url = self.server_auth(url, pa_token)
        ts_dict = {
            'project': {
                'key': project_key},
            'summary': (
                'Simulink Test Suite ' +
                ts_name),
            'issuetype': {
                'name': 'Test Set'},
            'description': (
                'Test Set created automatically for Simulink Test Suite ' +
                ts_name)}
        new_test_suite = self.create_issue_handle_ratelim(server, ts_dict)
        print('Successfully created Test Set with ID ' + new_test_suite.key)
        return new_test_suite.key


    def add_test_to_test_plan(self, testcase_key, testplan_key, url, pa_token):
        """Function to add test cases to a specified test plan"""
        assoctestcaseurl = url + '/rest/raven/1.0/api/testplan/' + testplan_key + '/test'
        assoctestcasejson = {'add': [testcase_key]}
        bearerauth = {'Authorization': 'Bearer ' + pa_token}
        response_code = self.post_request_handle_ratelim(
            assoctestcaseurl, bearerauth, assoctestcasejson)
        if str(response_code).startswith('2'):
            print('Successfully added Test Case ' +
                testcase_key + ' to Test Plan ' + testplan_key)
        else:
            print(
                'Error ' +
                str(response_code) +
                ' updating Test Plan ' +
                testplan_key +
                ' with Test Case ' +
                testcase_key)


    def create_test_case(self, url, pa_token, project_key, testcase_name, testcase_desc,
                        model_name, test_type, test_suite_id):
        """Function to create XRAY Test Case from Simulink Test Case"""
        # look for file specifying model-specific storage path
        testcase_path = self.get_model_xray_repo_path(model_name)
        pid_val = self.get_pid(model_name)
        # connect to server
        server, url = self.server_auth(url, pa_token)
        if len(testcase_desc) < 10:
            testcase_desc = (test_type + ' Test Case automatically created for '
                            + model_name + ' Simulink model.')
        # create issue
        if 'jira-test' not in url:
            test_type_xray = self.get_custom_field_option(server, '10823')  # 10600
            test_design_technique = self.get_custom_field_option(
                server, '26573')  # 16112
            test_design_technique_subitem = self.get_custom_field_option(
                server, '27002')
            test_method = self.get_custom_field_option(server, '29812')  # 16113
            det_event_rating_field = self.get_custom_field_option(
                server, '26540')  # 16110
            if test_type == 'SIL':
                test_target_level = self.get_custom_field_option(
                    server, '19203')  # 14701
            else:
                test_target_level = self.get_custom_field_option(
                    server, '19202')  # 14701
            tc_dict = {
                'project': {'key': project_key},
                'summary': ('Simulink ' + test_type + ' Test: ' + testcase_name +
                            ' for ' + model_name + ' Model'),
                'description': testcase_desc,
                'issuetype': {'name': 'Test'},
                'priority': {'name': 'Medium'},
                'customfield_10600': {'value': test_type_xray.value},
                'customfield_10611': testcase_path,  # XRAY Test Repository Path
                'customfield_16112': {'id': test_design_technique.id,
                                    'child': {'id': test_design_technique_subitem.id}},
                'customfield_16113': {'value': test_method.value},
                'customfield_14701': [{'id': str(test_target_level.id)}],
                'customfield_10609': [pid_val],
                'customfield_16110': {'value': det_event_rating_field.value},
                'customfield_10607': [test_suite_id],
                'customfield_16115': ['1'],  # Offcycle: None for PCDS Gateway
                'customfield_14056': {'value': 'Automated'}
            }
        else:
            print('WARNING: Creating Test Case in non-production Jira Instance!!')
            test_type_xray = self.get_custom_field_option(server, '30894')  # 10600
            tc_dict = {
                'project': {'key': project_key},
                'summary': ('Simulink ' + test_type + ' Test: ' + testcase_name +
                            ' for ' + model_name + ' Model'),
                'description': testcase_desc,
                'issuetype': {'name': 'Test'},
                'priority': {'name': 'Medium'},
                'customfield_10600': {'value': test_type_xray.value},
                'customfield_10611': testcase_path,
                'customfield_10609': [pid_val],
                'customfield_14056': {'value': 'Automated'}
            }
        new_test_case = self.create_issue_handle_ratelim(server, tc_dict)
        print('Successfully created Test Case with ID ' + new_test_case.key)
        if pid_val != '':
             self.add_test_to_test_plan(new_test_case.key, pid_val, url, pa_token)
        if 'jira-test' not in url:
            # Transition to Approved state.
            self.transition_to_approved(server, new_test_case)
        else:
            server.transition_issue(new_test_case, 'Start Progress')
        print('Sucessfully transitioned Test Case ' +
              new_test_case.key + ' to Approved State.')
        return new_test_case.key
        # return tc_dict, pid_val

    def add_test_cases_to_test_execution(
            self,
            url,
            pa_token,
            new_test_execution,
            testcase_keys):
        """Function to add Test Cases to Test Execution, with Retry handling logic."""
        assoctestcaseurl = url + '/rest/raven/1.0/api/testexec/' + \
            new_test_execution.key + '/test'
        assoctestcasejson = {'add': testcase_keys}
        bearerauth = {'Authorization': 'Bearer ' + pa_token}
        response_code = self.post_request_handle_ratelim(
            assoctestcaseurl, bearerauth, assoctestcasejson)
        if str(response_code).startswith('2'):
            print('Successfully added Test Cases to Test Execution')
        else:
            print(
                'Error ' +
                str(response_code) +
                ' updating Test Execution ' +
                new_test_execution.key +
                ' with test cases.')


    def create_test_execution(
            self,
            url,
            pa_token,
            project_key,
            testfile_name,
            model_name,
            testcase_keys,
            test_type):
        """Function to create XRAY Test Execution from run of Simulink Test Case"""
        # connect to server
        pid_val = self.get_pid(model_name)
        server, url = self.server_auth(url, pa_token)
        for tc_key in testcase_keys:
            tc = self.get_issue_handle_ratelim(server, tc_key)
            self.transition_to_in_progress(server, tc)
        if os.environ['CI_PIPELINE_SOURCE'] == 'merge_request_event':
            mr_url = os.environ['CI_PROJECT_URL'] + \
                '/-/merge_requests/' + os.environ['CI_MERGE_REQUEST_IID']
            desc = (test_type + ' Test Execution for ' + testfile_name +
                    ' Simulink Test File, executed on branch ' +
                    os.environ['CI_MERGE_REQUEST_SOURCE_BRANCH_NAME'] +
                    ' by Assignee ' + os.environ['GITLAB_USER_NAME'] +
                    ' (' + os.environ['GITLAB_USER_EMAIL'] + '), see GitLab MR: ' +
                    mr_url + ' for more information about this change.')
        else:
            desc = (
                test_type +
                ' Test Execution for ' +
                testfile_name +
                ' Simulink Test File, executed on branch ' +
                os.environ['CI_COMMIT_REF_NAME'] +
                ' by Assignee ' +
                os.environ['GITLAB_USER_NAME'] +
                ' (' +
                os.environ['GITLAB_USER_EMAIL'] +
                '), see GitLab Commit ID: ' +
                os.environ['CI_COMMIT_SHA'] +
                ' for more information about this change. ' +
                'The type of pipeline execution was: ' +
                os.environ['CI_PIPELINE_SOURCE'] +
                '.')
        if test_type.upper() == 'MIL':
            tams_key = 'TAMS-1665'
        else:
            tams_key = 'TAMS-1666'
        te_dict = {
            'project': {'key': project_key},
            'summary': ('Automated ' + test_type + ' Test Execution of: ' +
                        testfile_name + ' for ' + model_name + ' Model'),
            'description': desc,
            'issuetype': {'name': 'Test Execution'},
            'customfield_10627': [pid_val],
            'customfield_15004': tams_key
        }
        if 'jira-test' in url:
            print('WARNING: Creating Test Case in non-production Jira Instance!!')
        new_test_execution = self.create_issue_handle_ratelim(server, te_dict)

        # Associate Test Cases being executed with Test Execution
        self.add_test_cases_to_test_execution(
            url, pa_token, new_test_execution, testcase_keys)
        # Transition TCs to Approved
        for tc_key in testcase_keys:
            tc = self.get_issue_handle_ratelim(server, tc_key)
            self.transition_to_approved(server, tc)
        return new_test_execution.key


    def update_test_run(
            self,
            url,
            pa_token,
            test_case_key,
            test_execution_key,
            test_status):
        """Function to add a Test Run for a given Test Case and Test Execution"""
        # Get Test Run for given Test Case and Test Execution
        url = self.preprocess_url(url)
        bearerauth = {'Authorization': 'Bearer ' + pa_token}
        req_url = url + '/rest/raven/2.0/api/testrun/?testExecIssueKey=' + \
            test_execution_key + '&testIssueKey=' + test_case_key
        tr_req = self.get_request_handle_ratelim(req_url, bearerauth)
        testrun_id = tr_req.json()['id']
        # Update Test Run Status
        if test_status.upper() not in [
            'PASS',
            'FAIL',
            'TODO',
            'ABORTED',
                'EXECUTING']:
            raise ValueError(
                'Value of test_status must be PASS, FAIL, TODO, ABORTED or EXECUTING.')

        req_url = url + '/rest/raven/1.0/api/testrun/' + str(testrun_id) + \
            '/status?status=' + test_status.upper()
        response_code = self.put_request_handle_ratelim(req_url, bearerauth)
        if str(response_code).startswith('2'):
            print('Successfully updated Test Run to ' + test_status.upper() +
                ' for Test Case: ' + test_case_key)
        else:
            print(
                'Error ' +
                str(response_code) +
                ' updating Test Execution ' +
                test_execution_key +
                ' for Test Case ' +
                test_case_key +
                ' with correct status.')


    def detect_testcase_changes(
            self,
            url,
            pa_token,
            testcase_key,
            testcase_name,
            model_name,
            sl_testcase_desc,
            test_type,
            test_suite_id):
        """Function to detect changes in Simulink Test Case and update XRAY accordingly."""
        summary_string = ('Simulink ' + test_type + ' Test: '
                        + testcase_name + ' for ' + model_name + ' Model')
        update_list = []
        update_dict = {}
        # connect to server
        server, url = self.server_auth(url, pa_token)
        testcase_obj = self.get_issue_handle_ratelim(server, testcase_key)

        # Check Test Case Issue State, Move to In Progress to Update
        self.transition_to_in_progress(server, testcase_obj)

        testcase_summary = testcase_obj.fields.summary
        testcase_desc = testcase_obj.fields.description
        if 'jira-test' not in url:
            xray_path = testcase_obj.fields.customfield_10611
            pids_from_xray = testcase_obj.fields.customfield_10609
            test_suite_from_xray = testcase_obj.fields.customfield_10607
            der_from_xray = testcase_obj.fields.customfield_16110
            pcds_from_xray = testcase_obj.fields.customfield_16115
        else:
            xray_path = ''
            pids_from_xray = ''

        if testcase_summary is not summary_string:
            update_dict['summary'] = summary_string
            update_list.append('Summary')
        if testcase_desc is not None:
            if len(testcase_desc) < 10:
                if sl_testcase_desc is not None:
                    if len(sl_testcase_desc) < 10:
                        updated_desc = (
                            test_type +
                            ' Test Case automatically created for ' +
                            model_name +
                            ' Simulink model.')
                    else:
                        updated_desc = sl_testcase_desc
                else:
                    updated_desc = (
                        test_type +
                        ' Test Case automatically created for ' +
                        model_name +
                        ' Simulink model.')
                update_dict['description'] = updated_desc
                update_list.append('Description')
        else:
            updated_desc = (test_type + ' Test Case automatically created for '
                            + model_name + ' Simulink model.')
            update_dict['description'] = updated_desc
            update_list.append('Description')

        testcase_path = self.get_model_xray_repo_path(model_name)
        pid_val = self.get_pid(model_name)
        if (xray_path is not testcase_path) and testcase_path != '' and 'jira-test' not in url:
            update_dict['customfield_10611'] = testcase_path
            update_list.append('Repository Path')
        elif testcase_path == '':
            print('Test Case Path is blank - not updating.')

        if pid_val not in pids_from_xray and 'jira-test' not in url and len(
                pid_val) != 0:
            update_dict['customfield_10611'] = pids_from_xray.append(pid_val)
            update_list.append('PID')
        elif testcase_path == '' or len(pid_val) == 0:
            print('PID Ticket is blank or missing - not updating.')

        der_custfield = self.get_custom_field_option(server, '26540')
        if der_from_xray is None:
            update_dict['customfield_106110'] = {'id': str(der_custfield.id),
                                                'value': der_custfield.value}
            update_list.append('DER')

        tsfound = False
        for item in test_suite_from_xray:
            if item == test_suite_id:
                tsfound = True
        if not tsfound:
            # Associate Test Cases being executed with Test Execution
            assoctestcaseurl = url + '/rest/raven/1.0/api/testset/' + test_suite_id + '/test'
            assoctestcasejson = {'add': [testcase_key]}
            bearerauth = {'Authorization': 'Bearer ' + pa_token}
            response_code = self.post_request_handle_ratelim(
                assoctestcaseurl, bearerauth, assoctestcasejson)
            if str(response_code).startswith('2'):
                update_list.append('Test Suite')
            else:
                print(
                    'Error ' +
                    str(response_code) +
                    ' updating Test Suite ' +
                    test_suite_id +
                    ' with test cases.')

        if pcds_from_xray is None:
            try:
                self.add_field_value_handle_ratelim(
                    testcase_obj, 'customfield_16115', '1')
                update_list.append('PCDS Gateway')
            except JIRAError as e:
                print('Failed to update PCDS Gateway of ' + testcase_key + '.')
                print(e)

        # Perform issue update in one call
        if len(update_dict) > 0:
            self.update_issue_handle_ratelim(testcase_obj, update_dict)
        # Transition Issue to Approved State
        self.transition_to_approved(server, testcase_obj)

        print('Successfully Updated ' + ','.join(update_list) +
            ' for Test Case ' + testcase_key)
    